package domain;

import java.util.*;

import junit.framework.*;
import static test.CollectionsTestUtil.*;

public class GameTest extends TestCase {
   private static final int SMALL = 10;
   private static final int BIG = 20;

   private Game game;
   private Deck deck;
   private Player player1;
   private Player player2;
   private Player player3;

   protected void setUp() {
      game = new Game();
      game.setBlinds(SMALL, BIG);

      player1 = PlayerTest.createPlayer(1);
      player2 = PlayerTest.createPlayer(2);
      player3 = PlayerTest.createPlayer(3);
   }

   public void testCreate() {
      assertTrue(game.players().isEmpty());
   }

   public void testAddSinglePlayer() {
      Player player1 = PlayerTest.createPlayer(1);
      game.add(player1);
      assertCollection(game.players(), player1);
   }

   public void testAddMaximumNumberOfPlayers() {
      Player[] expected = new Player[Game.CAPACITY];
      for (int i = 0; i < Game.CAPACITY; i++) {
         Player player = PlayerTest.createPlayer(i);
         expected[i] = player;
         game.add(player);
      }
      assertCollection(game.players(), expected);
   }

   public void testDealCompleteHand() {
      addTwoPlayers();
      game.setButton(2);
      startHand();

      Card[] hole = deck.top(4);
      game.dealHoleCards();

      assertHoleCards(player1, hole, 0, 2);
      assertHoleCards(player2, hole, 1, 3);

      Card[] community = deck.top(3 * HandTest.BURN + 5);

      game.dealFlop();
      game.dealTurn();
      game.dealRiver();
      assertCollection(game.community(), community[1], community[2],
            community[3], community[5], community[7]);
   }

   private void startHand() {
      game.startHand();
      deck = game.deck();
   }

   public void testHandFlow() {
      addThreePlayers();
      game.setButton(3);

      game.startHand();
      Card[] deck1 = captureEntireDeck();
      dealAllCardsInHand();
      game.stopHand();
      assertHandCompletion(1);

      game.startHand();
      Card[] deck2 = captureEntireDeck();
      dealAllCardsInHand();
      game.stopHand();
      assertHandCompletion(2);

      game.startHand();
      Card[] deck3 = captureEntireDeck();
      dealAllCardsInHand();
      game.stopHand();
      assertHandCompletion(3);

      assertFalse(areEqual(deck1, deck2, deck3));
   }

   private void assertHandCompletion(int expectedButtonPosition) {
      assertEquals(expectedButtonPosition, game.buttonPosition());
      assertNoCardsOut();
   }

   private Card[] captureEntireDeck() {
      return game.deck().top(Deck.SIZE);
   }

   private boolean areEqual(Card[]... decks) {
      for (int i = 0; i < decks.length - 1; i++)
         if (!Arrays.equals(decks[i], decks[i + 1]))
            return false;
      return true;
   }

   private void assertNoCardsOut() {
      for (Player player: game.players())
         assertTrue(player.holeCards().isEmpty());
      assertTrue(game.community().isEmpty());
   }

   private void dealAllCardsInHand() {
      game.dealHoleCards();
      game.dealFlop();
      game.dealTurn();
      game.dealRiver();
   }

   private void addThreePlayers() {
      game.add(player1);
      game.add(player2);
      game.add(player3);
   }

   private void addTwoPlayers() {
      game.add(player1);
      game.add(player2);
   }

   private void assertHoleCards(Player player, Card[] hole, int... indices) {
      Card[] cards = new Card[indices.length];
      for (int i = 0; i < indices.length; i++)
         cards[i] = hole[indices[i]];
      DeckTest.assertCardsDealt(deck, cards);
      assertCollection(player.holeCards(), cards);
   }
}