package domain;

import static test.CollectionsTestUtil.*;
import junit.framework.*;

public class HandTest extends TestCase {
   static final int BURN = 1;
   private static final int SMALL = 10;
   private static final int BIG = 20;

   private Hand hand;
   private Player player1;
   private Player player2;
   private Player player3;
   private Ring ring;
   private Deck deck;

   protected void setUp() {
      player1 = PlayerTest.createPlayer(1);
      player2 = PlayerTest.createPlayer(2);
      player3 = PlayerTest.createPlayer(3);

      ring = new Ring();
      ring.add(player1, player2);
   }

   public void testCreate() {
      startHand(2);
      assertSame(ring, hand.ring());
   }

   public void testDealCompleteHand() {
      startHand(2);

      deck = hand.deck();
      Card[] hole = deck.top(4);
      hand.dealHoleCards();

      assertHoleCards(player1, hole, 0, 2);
      assertHoleCards(player2, hole, 1, 3);

      int remaining = Deck.SIZE - hole.length;
      assertEquals(remaining, deck.cardsRemaining());
      Card[] flop = deck.top(BURN + 3);

      hand.dealFlop();
      remaining -= flop.length;
      assertCardsDealt(remaining, flop);
      assertCollection(hand.community(), flop[1], flop[2], flop[3]);

      Card[] turn = deck.top(BURN + 1);
      hand.dealTurn();
      remaining -= turn.length;
      assertCardsDealt(remaining, turn);
      assertCollection(hand.community(), flop[1], flop[2], flop[3], turn[1]);

      Card[] river = deck.top(BURN + 1);
      hand.dealRiver();
      remaining -= river.length;
      assertCardsDealt(remaining, river);
      assertCollection(hand.community(), flop[1], flop[2], flop[3], turn[1],
            river[1]);

      hand.stop();
      assertNoCardsOut();
   }

   public void testBlinds() {
      ring.add(player3);

      startHand(3);
      assertEquals(PlayerTest.INITIAL_STAKE - SMALL, player1.chipCount());
      assertEquals(PlayerTest.INITIAL_STAKE - BIG, player2.chipCount());
      assertEquals(PlayerTest.INITIAL_STAKE, player3.chipCount());
      finishHand();

      startHand(1);
      assertEquals(PlayerTest.INITIAL_STAKE - SMALL, player1.chipCount());
      assertEquals(PlayerTest.INITIAL_STAKE - BIG - SMALL, player2.chipCount());
      assertEquals(PlayerTest.INITIAL_STAKE - BIG, player3.chipCount());
      finishHand();

      startHand(2);
      assertEquals(PlayerTest.INITIAL_STAKE - SMALL - BIG, player1.chipCount());
      assertEquals(PlayerTest.INITIAL_STAKE - BIG - SMALL, player2.chipCount());
      assertEquals(PlayerTest.INITIAL_STAKE - BIG - SMALL, player3.chipCount());
      finishHand();
   }

   private void finishHand() {
      dealAllCardsInHand();
      hand.stop();
   }

   private void assertNoCardsOut() {
      for (Player player: ring)
         assertTrue(player.holeCards().isEmpty());
      assertTrue(hand.community().isEmpty());
   }

   public void testDealOrderStartsFromButton() {
      startHand(1);

      Card[] hole = deck.top(4);
      dealAllCardsInHand();
      assertHoleCards(player1, hole, 1, 3);
      assertHoleCards(player2, hole, 0, 2);
      hand.stop();

      startHand(2);
      hole = deck.top(4);
      dealAllCardsInHand();
      assertHoleCards(player1, hole, 0, 2);
      assertHoleCards(player2, hole, 1, 3);
      hand.stop();
   }

   private void startHand(int buttonPosition) {
      ring.setButton(buttonPosition);
      hand = new Hand(ring, SMALL, BIG);
      deck = hand.deck();
   }

   private void dealAllCardsInHand() {
      hand.dealHoleCards();
      hand.dealFlop();
      hand.dealTurn();
      hand.dealRiver();
   }

   private void assertHoleCards(Player player, Card[] hole, int... indices) {
      Card[] cards = new Card[indices.length];
      for (int i = 0; i < indices.length; i++)
         cards[i] = hole[indices[i]];
      DeckTest.assertCardsDealt(deck, cards);
      assertCollection(player.holeCards(), cards);
   }

   private void assertCardsDealt(int remaining, Card[] turn) {
      assertDeckCount(remaining);
      DeckTest.assertCardsDealt(deck, turn);
   }

   private void assertDeckCount(int expected) {
      assertEquals(expected, deck.cardsRemaining());
   }
}